import tap from 'tap'
import dotenv from 'dotenv'
import Fastify from 'fastify'
import { request } from 'undici'

import fastifyHasura from '../plugin.js'

dotenv.config()

tap.test('fastify-hasura', async t => {
  const fastify = Fastify()
  const endpoint = process.env.HASURA_GRAPHQL_ENDPOINT

  fastify.register(fastifyHasura, {
    endpoint,
    admin_secret: process.env.HASURA_GRAPHQL_ADMIN_SECRET,
    api_secret: process.env.API_SECRET
  })

  await fastify.ready()

  // const {
  //   statusCode,
  //   headers,
  //   trailers,
  //   body
  // } = await request(endpoint.replace('v1/graphql', 'healthz'))

  // Test hasura plugin correctly loaded
  t.type(
    fastify.hasura.graphql,
    'function',
    'fastify.hasura.graphql is function'
  )

  // Test hasura endpoint health response
  const { statusCode } = await request(endpoint.replace('v1/graphql', 'healthz'))
  t.equal(statusCode, 200, `${endpoint} healthy`)

  const graphql = async (query, variables) => {
    const {
      statusCode,
      headers,
      trailers,
      body
    } = await request(endpoint + 'xx', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json; charset=utf-8', 'x-hasura-admin-secret': 'aymara' },
      body: JSON.stringify({
        query
      })
    })
    console.log({
      statusCode,
      headers,
      trailers,
      body
    })
    if (statusCode !== 200) throw new Error('Wrong request')

    return await body.json()
  }
  const data = await graphql(`#graphql
    mutation insertLog($object: logs_insert_input!) {
      insert_logs_one(object: $object) {
        id
      }
    }
  `)
  console.log('data', data.errors)

  // Throw nice error
  try {
    await fastify.hasura.graphql(
      `#graphql
        query test {
          test
        }
      `
    )
  } catch (err) {
    t.equal(err.statusCode, 200, 'Return statusCode 200')
    t.equal(
      err.message,
      `field "test" not found in type: 'query_root'`,
      'Throw nice error'
    )
  }

  await fastify.close()
})
